/*
 * Copyright 2016 - 2024 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#include "inserttilescommand.hpp"

namespace nostalgia::core {

core::InsertTilesCommand::InsertTilesCommand(
		TileSheet &img,
		TileSheet::SubSheetIdx idx,
		std::size_t tileIdx,
		std::size_t tileCnt) noexcept:
		m_img(img),
		m_idx(std::move(idx)) {
	const unsigned bytesPerTile = m_img.bpp == 4 ? PixelsPerTile / 2 : PixelsPerTile;
	m_insertPos = tileIdx * bytesPerTile;
	m_insertCnt = tileCnt * bytesPerTile;
	m_deletedPixels.resize(m_insertCnt);
	// copy pixels to be erased
	{
		auto &s = getSubSheet(m_img, m_idx);
		auto &p = s.pixels;
		auto dst = m_deletedPixels.data();
		auto src = &p[p.size() - m_insertCnt];
		const auto sz = m_insertCnt * sizeof(decltype(p[0]));
		ox::memcpy(dst, src, sz);
	}
}

ox::Error InsertTilesCommand::redo() noexcept {
	auto &s = getSubSheet(m_img, m_idx);
	auto &p = s.pixels;
	auto dstPos = m_insertPos + m_insertCnt;
	auto const dst = &p[dstPos];
	auto const src = &p[m_insertPos];
	ox::memmove(dst, src, p.size() - dstPos);
	ox::memset(src, 0, m_insertCnt * sizeof(decltype(p[0])));
	return {};
}

ox::Error InsertTilesCommand::undo() noexcept {
	auto &s = getSubSheet(m_img, m_idx);
	auto &p = s.pixels;
	auto const srcIdx = m_insertPos + m_insertCnt;
	auto const src = &p[srcIdx];
	auto const dst1 = &p[m_insertPos];
	auto const dst2 = &p[p.size() - m_insertCnt];
	auto const sz = p.size() - srcIdx;
	ox::memmove(dst1, src, sz);
	ox::memcpy(dst2, m_deletedPixels.data(), m_deletedPixels.size());
	return {};
}

int InsertTilesCommand::commandId() const noexcept {
	return static_cast<int>(CommandId::InsertTile);
}

TileSheet::SubSheetIdx const&InsertTilesCommand::subsheetIdx() const noexcept {
	return m_idx;
}

}
